@@ -2,7 +2,7 @@
 ######################################################################
 ### SNMP_util -- SNMP utilities using SNMP_Session.pm and BER.pm
 ######################################################################
-### Copyright (c) 1998-2002, Mike Mitchell.
+### Copyright (c) 1998-2005, Mike Mitchell.
 ###
 ### This program is free software; you can redistribute it under the
 ### "Artistic License" included in this distribution (file "Artistic").
@@ -11,22 +11,23 @@
 ###
 ### Contributions and fixes by:
 ###
-### Tobias Oetiker <oetiker@ee.ethz.ch>:  Basic layout
+### Tobias Oetiker <tobi.ch>:  Basic layout
 ### Simon Leinen <simon@switch.ch>: SNMP_session.pm/BER.pm
 ### Jeff Allen <jeff.allen@acm.org>: length() of undefined value
 ### Johannes Demel <demel@zid.tuwien.ac.at>: MIB file parse problem
 ### Simon Leinen <simon@switch.ch>: more OIDs from Interface MIB
 ### Jacques Supcik <supcik@ip-plus.net>: Specify local IP, port
-### Tobias Oetiker <oetiker@ee.ethz.ch>: HASH as first OID to set SNMP options
+### Tobias Oetiker <tobi.ch>: HASH as first OID to set SNMP options
 ### Simon Leinen <simon@switch.ch>: 'undefined port' bug
 ### Daniel McDonald <dmcdonald@digicontech.com>: request for getbulk support
-### Laurent Girod <girod.laurent@pmintl.ch>: code for snmpwalkhash
+### Laurent Girod <it.fb@net2000.ch>: code for snmpwalkhash
 ### Ian Duplisse <i.duplisse@cablelabs.com>: MIB parsing suggestions
 ### Jakob Ilves <jakob.ilves@oracle.com>: return_array_refs for snmpwalk()
 ### Valerio Bontempi <v.bontempi@inwind.it>: IPv6 support
 ### Lorenzo Colitti <lorenzo@colitti.com>: IPv6 support
 ### Joerg Kummer <JOERG.KUMMER@Roche.COM>: TimeTicks support in snmpset()
 ### Christopher J. Tengi <tengi@CS.Princeton.EDU>: Gauge32 support in snmpset()
+### Nicolai Petri <nicolai@catpipe.net>: hashref passing for snmpwalkhash()
 ######################################################################
 
 package SNMP_util;
@@ -38,11 +39,11 @@ use vars qw(@ISA @EXPORT $VERSION);
 use Exporter;
 use Carp;
 
-use BER "0.95";
-use SNMP_Session "0.97";
+use BER "1.02";
+use SNMP_Session "1.00";
 use Socket;
 
-$VERSION = '0.99';
+$VERSION = '1.06';
 
 @ISA = qw(Exporter);
 
@@ -340,6 +341,7 @@ $SNMP_util::Debug = 0;
 $SNMP_util::CacheFile = "OID_cache.txt";
 $SNMP_util::CacheLoaded = 0;
 $SNMP_util::Return_array_refs = 0;
+$SNMP_util::Return_hash_refs = 0;
 
 srand(time + $$);
 
@@ -436,8 +438,9 @@ sub snmpopen ($$$) {
       foreach $type (keys %$opts) {
 	if ($type eq 'return_array_refs') {
 	  $SNMP_util::Return_array_refs = $opts->{$type};
-	}
-	else {
+	} elsif ($type eq 'return_hash_refs') {
+	  $SNMP_util::Return_hash_refs = $opts->{$type};
+	} else {
 	  if (exists $SNMP_util::Session->{$type}) {
 	    if ($type eq 'timeout') {
 	      $SNMP_util::Session->set_timeout($opts->{$type});
@@ -493,7 +496,7 @@ sub snmpget ($@) {
       my $tempo = pretty_print($value);
       push @retvals, $tempo;
     }
-    return(@retvals);
+    return wantarray ? @retvals : $retvals[0];
   }
   $var = join(' ', @vars);
   carp "SNMPGET Problem for $var on $host\n"
@@ -537,7 +540,7 @@ sub snmpgetnext ($@) {
       my $tempv = pretty_print($value);
       push @retvals, "$tempo:$tempv";
     }
-    return (@retvals);
+    return wantarray ? @retvals : $retvals[0];
   } else {
     $var = join(' ', @vars);
     carp "SNMPGETNEXT Problem for $var on $host\n"
@@ -569,7 +572,7 @@ sub snmpwalk_flg ($$@) {
   my($got, @nnoid, $noid, $ok, $ix, @avars);
   my $session;
   my(%soid);
-  my(%done, %rethash);
+  my(%done, %rethash, $h_ref);
 
   $session = &snmpopen($host, 0, \@vars);
   if (!defined($session)) {
@@ -578,6 +581,8 @@ sub snmpwalk_flg ($$@) {
     return undef;
   }
 
+  $h_ref = (ref $vars[$#vars] eq "HASH") ? pop(@vars) : \%rethash;
+
   @enoid = toOID(@vars);
   return undef unless defined $enoid[0];
 
@@ -585,9 +590,9 @@ sub snmpwalk_flg ($$@) {
   #
   # Create/Refresh a reversed hash with oid -> name
   #
-  if (defined($hash_sub) && $RevNeeded) {
-    %revOIDS = reverse %SNMP_util::OIDS;
-    $RevNeeded = 0;
+  if (defined($hash_sub) && ($RevNeeded)) {
+      %revOIDS = reverse %SNMP_util::OIDS;
+      $RevNeeded = 0;
   }
 
   $got = 0;
@@ -645,7 +650,7 @@ sub snmpwalk_flg ($$@) {
       } else {
 	# IlvJa
 	#
-	# The walk for variable $var[$ix] has been finished as
+	# The walk for variable $vars[$ix] has been finished as
 	# $nnoid[$ix] no longer is in the $avar[$ix] OID tree.
 	# So we exclude this variable from further requests.
 
@@ -691,14 +696,14 @@ sub snmpwalk_flg ($$@) {
 	    }
 	  }	
 	  if (length($tempo) && exists($revOIDS{$tempo})) {
-	    $tempo = $revOIDS{$tempo} . $inst;
+	    $var = $revOIDS{$tempo};
 	  } else {
-	    $tempo = pretty_print($oid);
+	    $var = pretty_print($oid);
 	  }
 	  #
 	  # call hash_sub
 	  #
-	  &$hash_sub(\%rethash, $host, $revOIDS{$tempo}, $tempo, $inst,
+	  &$hash_sub($h_ref, $host, $var, $tempo, $inst,
 			$tempv, $upo);
 	} else {
 	  if ($SNMP_util::Return_array_refs) {
@@ -729,7 +734,8 @@ sub snmpwalk_flg ($$@) {
   }
   if ($got) {
     if (defined($hash_sub)) {
-    	return (%rethash)
+	return ($h_ref) if ($SNMP_util::Return_hash_refs);
+    	return (%$h_ref);
     } else {
     	return (@retvals);
     }
@@ -828,7 +834,7 @@ sub snmpset($@) {
       my $tempo = pretty_print($value);
       push @retvals, $tempo;
     }
-    return (@retvals);
+    return wantarray ? @retvals : $retvals[0];
   }
   return undef;
 }
@@ -1096,7 +1102,7 @@ sub Check_OID ($) {
     $tmp = $&;
     $tmpv = $tmp;
     for (;;) {
-      last if defined($SNMP_util::OIDS{$tmpv});
+      last if exists($SNMP_util::OIDS{$tmpv});
       last if !($tmpv =~ s/^[^\.]*\.//);
     }
     $oid = $SNMP_util::OIDS{$tmpv};
@@ -1231,9 +1237,13 @@ sub snmpMIB_to_OID ($) {
 	    ($tmp, $val) = split(' ', $val, 2);
 	    if ($tmp =~ /([\w\-]+)\((\d+)\)/) {
 	      $tmp = $1;
-	      $tmpv = "$SNMP_util::OIDS{$strt}.$2";
+	      if (exists($SNMP_util::OIDS{$strt})) {
+		$tmpv = "$SNMP_util::OIDS{$strt}.$2";
+	      } else {
+		$tmpv = $2;
+	      }
 	      $Link{$tmp} = $strt;
-	      if (!defined($prev{$tmp}) && defined($SNMP_util::OIDS{$tmp})) {
+	      if (!exists($prev{$tmp}) && exists($SNMP_util::OIDS{$tmp})) {
 		if ($tmpv ne $SNMP_util::OIDS{$tmp}) {
 		  $strt = "$strt.$tmp";
 		  $SNMP_util::OIDS{$strt} = $tmpv;
@@ -1248,7 +1258,7 @@ sub snmpMIB_to_OID ($) {
 	    }
 	  }
 
-	  if (!defined($SNMP_util::OIDS{$strt})) {
+	  if (!exists($SNMP_util::OIDS{$strt})) {
 	    if ($pass) {
 	      carp "snmpMIB_to_OID: $arg: \"$strt\" prefix unknown, load the parent MIB first.\n"
 		unless ($SNMP_Session::suppress_warnings > 1);
@@ -1257,8 +1267,10 @@ sub snmpMIB_to_OID ($) {
 	    }
 	  }
 	  $Link{$var} = $strt;
-	  $val = "$SNMP_util::OIDS{$strt}.$val";
-	  if (!defined($prev{$var}) && defined($SNMP_util::OIDS{$var})) {
+	  if (exists($SNMP_util::OIDS{$strt})) {
+	    $val = "$SNMP_util::OIDS{$strt}.$val";
+	  }
+	  if (!exists($prev{$var}) && exists($SNMP_util::OIDS{$var})) {
 	    if ($val ne $SNMP_util::OIDS{$var}) {
 	      $var = "$strt.$var";
 	    }
